@@ -1,4 +1,4 @@
-use clap::{App, Arg, crate_description, crate_name, crate_version, arg_enum, value_t};
+use clap::{App, AppSettings, Arg, crate_description, crate_name, crate_version, arg_enum, value_t};
 use remoteprocess::Pid;
 
 /// Options on how to collect samples from a python process
@@ -50,6 +50,8 @@ pub struct Config {
     pub dump_locals: u64,
     #[doc(hidden)]
     pub full_filenames: bool,
+    #[doc(hidden)]
+    pub lineno: LineNo,
 }
 
 arg_enum!{
@@ -76,6 +78,13 @@ pub enum RecordDuration {
     Seconds(u64)
 }
 
+#[derive(Debug, Clone, Eq, PartialEq, Copy)]
+pub enum LineNo {
+    NoLine,
+    FirstLineNo,
+    LastInstruction
+}
+
 impl Default for Config {
     /// Initializes a new Config object with default parameters
     #[allow(dead_code)]
@@ -86,7 +95,7 @@ impl Default for Config {
                duration: RecordDuration::Unlimited, native: false,
                gil_only: false, include_idle: false, include_thread_ids: false,
                hide_progress: false, capture_output: true, dump_json: false, dump_locals: 0, subprocesses: false,
-               full_filenames: false}
+               full_filenames: false, lineno: LineNo::LastInstruction }
     }
 }
 
@@ -182,7 +191,10 @@ impl Config {
             .arg(Arg::with_name("function")
                 .short("F")
                 .long("function")
-                .help("Aggregate samples by function name instead of by line number"))
+                .help("Aggregate samples by function's first line number, instead of current line number"))
+            .arg(Arg::with_name("nolineno")
+                .long("nolineno")
+                .help("Do not show line numbers"))
             .arg(Arg::with_name("threads")
                 .short("t")
                 .long("threads")
@@ -222,6 +234,13 @@ impl Config {
                 .long("json")
                 .help("Format output as JSON"));
 
+        let completions = clap::SubCommand::with_name("completions")
+            .about("Generate shell completions")
+            .setting(AppSettings::Hidden)
+            .arg(Arg::with_name("shell")
+                .possible_values(&clap::Shell::variants())
+                .help("Shell type"));
+
         // add native unwinding if appropiate
         #[cfg(unwind)]
         let record = record.arg(native.clone());
@@ -238,7 +257,7 @@ impl Config {
         #[cfg(not(target_os="freebsd"))]
         let dump = dump.arg(nonblocking.clone());
 
-        let matches = App::new(crate_name!())
+        let mut app = App::new(crate_name!())
             .version(crate_version!())
             .about(crate_description!())
             .setting(clap::AppSettings::InferSubcommands)
@@ -248,7 +267,8 @@ impl Config {
             .subcommand(record)
             .subcommand(top)
             .subcommand(dump)
-            .get_matches_from_safe(args)?;
+            .subcommand(completions);
+        let matches = app.clone().get_matches_from_safe(args)?;
         info!("Command line args: {:?}", matches);
 
         let mut config = Config::default();
@@ -269,6 +289,11 @@ impl Config {
             "top" => {
                 config.sampling_rate = value_t!(matches, "rate", u64)?;
             }
+            "completions" => {
+                let shell = value_t!(matches.value_of("shell"), clap::Shell).unwrap_or_else(|e| e.exit());
+                app.gen_completions_to(crate_name!(), shell, &mut std::io::stdout());
+                std::process::exit(0);
+            }
             _ => {}
         }
         config.command = subcommand.to_owned();
@@ -278,7 +303,7 @@ impl Config {
         config.python_program = matches.values_of("python_program").map(|vals| {
             vals.map(|v| v.to_owned()).collect()
         });
-        config.show_line_numbers = matches.occurrences_of("function") == 0;
+        config.show_line_numbers = matches.occurrences_of("nolineno") == 0;
         config.include_idle = matches.occurrences_of("idle") > 0;
         config.gil_only = matches.occurrences_of("gil") > 0;
         config.include_thread_ids = matches.occurrences_of("threads") > 0;
@@ -289,6 +314,11 @@ impl Config {
         config.dump_locals = matches.occurrences_of("locals");
         config.subprocesses = matches.occurrences_of("subprocesses") > 0;
         config.full_filenames = matches.occurrences_of("full_filenames") > 0;
+        config.lineno = if matches.occurrences_of("nolineno") > 0 { LineNo::NoLine } else if matches.occurrences_of("function") > 0 { LineNo::FirstLineNo } else { LineNo::LastInstruction };
+        if matches.occurrences_of("nolineno") > 0 && matches.occurrences_of("function") > 0 {
+            eprintln!("--function & --nolinenos can't be used together");
+            std::process::exit(1);
+        }
 
         config.capture_output = config.command != "record" || matches.occurrences_of("capture") > 0;
         if !config.capture_output {
